//+------------------------------------------------------------------+
//|                                                    Plot_News.mq4 |
//|                                                   David Louisson |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+

//+-------------------------------------------------------------------------------------+
//|   Version 1.0    05 Feb 2007                                                        |
//|   Update for v137  17 Jul 2009 -- by pips4life                                      |
//|                                                                                     |
//|   -- Reads news from NEWS.CSV file and plots appropriate objects on chart           |
//|                                                                                     |
//|   -- Various enhancements by Pips4Life                                              |
//|                                                                                     |
//+-------------------------------------------------------------------------------------+

//+-------------------------------------------------------------------------------------+
//|                                                                                     |
//| DISCLAIMER:                                                                         |
//|  This program is provided without any warranty, expressed or implied. The           |
//|  program may contain errors that result in wrong calculations and/or faulty         | 
//|  operation.  The program is not guaranteed to work with any past, present or        | 
//|  future version of MetaTrader, and no ongoing support is expressed or implied.      |
//|  The author(s) accept no liability whatsoever for any losses or damages to your     | 
//|  account, your computer, nor anything else, resulting from the use of this          |
//|  program or information.  The comments in this program and associated documents     |
//|  are for educational purposes only and should not be substituted for the advice     |
//|  of a professional broker. Use extreme caution.  Past performance and price action  | 
//|  does not guarantee similar performance and price action in the future.  Trading    | 
//|  Foreign Exchange (Forex) carries a high level of risk and may not be suitable      | 
//|  for all investors. There is a possibility that you could sustain a loss of all     |
//|  (100%!) or more of your investment.  Therefore, you should not invest money that   |
//|  you cannot afford to lose. You should be aware of all the risks associated with    |
//|  Forex trading and you accept all liabilities.                                      | 
//|                                                                                     |
//+-------------------------------------------------------------------------------------+

#property copyright "David Louisson"
#property link      "http://www.metaquotes.net"

#property indicator_chart_window

// NEWS.CSV file must exist in: \metatrader 4\experts\files
extern   string   FileName          = "news.csv";   // News file name
extern   int      VertSpacing       = 5;            // # pips spacing between symbols plotted against same bar
extern   int      SymbolSize        = 2;            // size of symbols being plotted on the chart
extern   bool     AutoRefresh       = TRUE;         // whether to refresh symbols/text once per minute
extern   int      AlertMins         = -1;           // display/sound alert X mins before the event (AutoRefresh must be TRUE); set to -1 to disable alerts
extern   bool     RemindToRunFFcal  = TRUE;         // On Sunday-Thursday, if no future FF arrows, display a REMINDER text

// Filters (set parameter to FALSE to suppress plot):
extern   datetime FromDate          = D'1990.01.01 00:00';
extern   datetime ThruDate          = D'2019.12.31 23:59';
extern   bool     Plot_High         = TRUE;         // High impact   = 3 (red plot)
extern   bool     Plot_Medium       = TRUE;         // Medium impact = 2 (orange plot)
extern   bool     Plot_Low          = TRUE;         // Low impact    = 1 (yellow plot)
extern   bool     Plot_USD          = TRUE;         // USD = 1 (wingding character #140)
extern   bool     Plot_CAD          = TRUE;         // CAD = 2 (wingding character #141)
extern   bool     Plot_EUR          = TRUE;         // EUR = 3 (wingding character #142)
extern   bool     Plot_GBP          = TRUE;         // GBP = 4 (wingding character #143)
extern   bool     Plot_CHF          = TRUE;         // CHF = 5 (wingding character #144)
extern   bool     Plot_JPY          = TRUE;         // JPY = 6 (wingding character #145)
extern   bool     Plot_AUD          = TRUE;         // AUD = 7 (wingding character #146)
extern   bool     Plot_NZD          = TRUE;         // NZD = 8 (wingding character #147)
extern   bool     Plot_CNY_OTHERS   = TRUE;         // CNY_OTHERS = 9 (wingding character #148)

// Text display
extern   int      DisplayPrior      = 0;
extern   int      DisplayNext       = 0;
extern   int      Window            = 0;
extern   int      xStart            = 0;
extern   int      yStart            = 0;

extern   bool     TimeZones         = FALSE;
extern   color    TimeZoneColor     = White;
extern   int      NZshift           = 12;  // 10, 12 ?
extern   int      AUshift           = 10;  // 8, 10 ?
extern   int      JPshift           = 8;   // 7, 8 ?
extern   int      EUshift           = 0;
extern   int      GBshift           = -1;
extern   int      USshift           = -5;  // -6, -5 ?

string   FontName          = "Courier New";
int      FontSize          = 9;
color    FontColor         = White;

int      handle, curr_min, prev_min;
bool     init_flag;

int      eCount;              // Event counter
datetime eTime[9999];         // Event date/time, format = yyyy.mm.dd<space>hh:mm
int      eNumber[9999];       // Event seq# (same date/time use ascending seq#s)
int      eWingding[9999];     // Wingding code for implicated currency, as above
int      eImpact[9999];       // 1=low impact; 2=medium impact; 3=high impact
string   eCurrency[9999];     // Currency/ies on whose charts object will display; ALL = all currencies 
string   ePeriods[9999];      // Time period settings on which object will display (M1,M5,M15,M30,H1,H4,D1,W1,MTH; 0=no, 1=yes)
string   eText1[9999];        // First descr line, format = hh:mm XXX Y "event description", where XXX=currency; Y=H/M/L impact
string   eText2[9999];        // Second descr line, shows Actual, Forecast, Previous values
int      eAlertTime[9999];    // Last time alert for this event sounded 

bool FLAG_FutureFFEventFound = FALSE;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()  {
  init_flag = true;
  delete_objects();
  read_data();
  main_process();
  display_reminder();
  prev_min = -1;
  init_flag = false;
  return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()  {
  delete_objects();
  return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()  {
  curr_min = Minute();
  if (AutoRefresh == true && curr_min != prev_min)  {
    delete_objects();
    read_data();
    main_process();
    display_reminder();
  }
  prev_min = curr_min;  
  return(0);
}

//+------------------------------------------------------------------+
//| read_data                                                        |
//+------------------------------------------------------------------+
void read_data()  {

  handle = FileOpen(FileName, FILE_CSV|FILE_READ,';');            // Open NEWS.CSV
  if(handle==0)  Comment("File "+FileName+" not found.");

  for(eCount = 0; !FileIsEnding(handle); eCount++)  {             // Read from NEWS.CSV; one line per event
    // Note: Do NOT prefix this first read with "if (!FileIsLineEnding...)" because it will HANG MT4 startup!!
    eTime[eCount]     = StrToTime(FileReadString(handle));        // Event date/time
    if (!FileIsLineEnding(handle)) eNumber[eCount]   = StrToInteger(FileReadString(handle));     // Event seq# (same date/time have ascending seq#s)
    if (!FileIsLineEnding(handle)) eWingding[eCount] = StrToInteger(FileReadString(handle));     // Wingding code denoting implicated currency
    if (!FileIsLineEnding(handle)) eImpact[eCount]   = StrToInteger(FileReadString(handle));     // Impact: 1=low; 2=medium; 3=High
    if (!FileIsLineEnding(handle)) eCurrency[eCount] = FileReadString(handle);                   // Currency/ies on whose charts object will display; ALL = all currencies
    if (!FileIsLineEnding(handle)) ePeriods[eCount]  = FileReadString(handle);                   // Time period settings on which object will display (M1,M5,M15,M30,H1,H4,D1,W1,MN; 0=no, 1=yes)
    if (!FileIsLineEnding(handle)) eText1[eCount]    = FileReadString(handle);                   // First line of description
    if (!FileIsLineEnding(handle)) eText2[eCount]    = FileReadString(handle);                   // Second line of description

    // The entire line is parsed until the EOL.
    while(!FileIsLineEnding(handle))  {eText2[eCount] = StringConcatenate(eText2[eCount],FileReadString(handle));}
    
    // This 2009-01-28 string replace section should no longer be necessary >= FFcal.exe version v1.32
    // Note: New function stringReplaceEveryMatch is appended at the end of this file.
    //eText2[eCount] = stringReplaceEveryMatch(eText2[eCount],"&lt","<" );
    //eText2[eCount] = stringReplaceEveryMatch(eText2[eCount],"&gt",">" );
    //eText2[eCount] = stringReplaceEveryMatch(eText2[eCount],"&amp","+" ); // A literal "&" is invisible, so use "=" instead.
    // End of bug fix 2009-01-28
    
    //    if (eWingding[eCount] == 0) eWingding[eCount] = 148;
        
    if (init_flag)  {
      if (AlertMins < 0) 
        eAlertTime[eCount] = 0;
      else
        eAlertTime[eCount] = eTime[eCount] - AlertMins * 60;
  } }        

  FileClose(handle);
/*
   int debug, i;
   debug=FileOpen("debug.txt", FILE_CSV|FILE_WRITE, '|');
   for( i=0; i<eCount; i++)
     FileWrite(debug,i,TimeToStr(eTime[i]),eNumber[i],eWingding[i],eImpact[i],eCurrency[i],eText1[i],eText2[i]);
   FileClose(debug);
*/
  return(0);
}

//+------------------------------------------------------------------+
//| main_process                                                     |
//+------------------------------------------------------------------+
void main_process()   {
  string objname, sym1, sym2, cmp1, cmp2;
  int    barno, dir1, i, LastEvent, Mins;
  double vertpos;

  FLAG_FutureFFEventFound = false;

  for(i=0; i<eCount; i++)  {

    if (eNumber[i] == 0 )      continue;                  // Invalid entry, or EOF
    
    if (eTime[i] < TimeCurrent())  LastEvent = i;       

    if (eTime[i] < FromDate) continue;                    // Skip if outside entered date range
    if (eTime[i] > ThruDate) continue;

    // Check if this event is a FF (ForexFactory) future event
    // The intent is to set the flag ONLY if a future *FF* event was found. Ignore user-defined events.
    // Another possible check to add to the logic below could be:
    //     Find the last word in eText1[eCount] (the Event Id#). If 1st-char is "*" skip it because it's user-defined.
    //     But if no leading "*", then use StrToInteger and check if > 3000. All FF events since 2007 are above 3000.

    string userEventString = "\034 \042";                   // User-defined events should contain:  doublequote(ascii=34), space, asterisk(ascii=42)
    //ALT: string userEventString = StringConcatenate( StringSetChar(" ", 0, 34), " ", StringSetChar(" ", 0, 42));
    if (eTime[i] > TimeCurrent()                            // If the event is in the future...
    && eNumber[i] > 0                                       // FF events are always positive.  User-defined events *should* be negative but may not be
    && StringFind( eText1[i], userEventString, 0) < 0       // If the event 1st-line does NOT contain the userEventString
       ) FLAG_FutureFFEventFound = TRUE;                    // TRUE means a future FF event was found.
  
    if (eImpact[i] == 1 && Plot_Low == FALSE)    continue;                     // Skip if impact outside entered values
    if (eImpact[i] == 2 && Plot_Medium == FALSE) continue;
    if (eImpact[i] == 3 && Plot_High == FALSE)   continue;
      
    if (eWingding[i] == 140 && Plot_USD == FALSE)   continue;                  // Skip if currency outside entered values
    if (eWingding[i] == 141 && Plot_CAD == FALSE)   continue;
    if (eWingding[i] == 142 && Plot_EUR == FALSE)   continue;
    if (eWingding[i] == 143 && Plot_GBP == FALSE)   continue;
    if (eWingding[i] == 144 && Plot_CHF == FALSE)   continue;
    if (eWingding[i] == 145 && Plot_JPY == FALSE)   continue;
    if (eWingding[i] == 146 && Plot_AUD == FALSE)   continue;
    if (eWingding[i] == 147 && Plot_NZD == FALSE)   continue;
    if (eWingding[i] == 148 && Plot_CNY_OTHERS == FALSE)   continue;

    if ( StringSubstr(ePeriods[i],0,1) == "0" && Period() == PERIOD_M1  )   continue;    // Skip if not to be shown on this period's chart
    if ( StringSubstr(ePeriods[i],1,1) == "0" && Period() == PERIOD_M5  )   continue;
    if ( StringSubstr(ePeriods[i],2,1) == "0" && Period() == PERIOD_M15 )   continue;
    if ( StringSubstr(ePeriods[i],3,1) == "0" && Period() == PERIOD_M30 )   continue;
    if ( StringSubstr(ePeriods[i],4,1) == "0" && Period() == PERIOD_H1  )   continue;
    if ( StringSubstr(ePeriods[i],5,1) == "0" && Period() == PERIOD_H4  )   continue;
    if ( StringSubstr(ePeriods[i],6,1) == "0" && Period() == PERIOD_D1  )   continue;
    if ( StringSubstr(ePeriods[i],7,1) == "0" && Period() == PERIOD_W1  )   continue;
    if ( StringSubstr(ePeriods[i],8,1) == "0" && Period() == PERIOD_MN1 )   continue;

    bool skipflag = FALSE;                                                          // Skip if currency outside entered values
    if (eCurrency[i] != "ALL")  {
      skipflag = TRUE;
      sym1 = StringSubstr(Symbol(),0,3);
      sym2 = StringSubstr(Symbol(),3,3);
      cmp1 = StringSubstr(eCurrency[i],0,3);
      if (StringLen(eCurrency[i]) > 3)  {
        cmp2 = StringSubstr(eCurrency[i],3,3);
        if ((cmp1 == "ALL" || cmp1 == sym1 || cmp1 == sym2) && (cmp2 == "ALL" || cmp2 == sym1 || cmp2 == sym2)) skipflag = FALSE;   
      }
      else  {
        if (cmp1 == "ALL" || cmp1 == sym1 || cmp1 == sym2) skipflag = FALSE;   
      }  
    }
    if (skipflag == TRUE) continue;  

    objname = eText1[i];                                                       // Set object name = first descr line
    barno = iBarShift(NULL,0,eTime[i],FALSE);                                  // Number of bars back from current

    dir1 = 1;
    if (eNumber[i] < 0) dir1 = -1;
      
    if (VertSpacing * eNumber[i] > 0) 
      vertpos = iLow(NULL,0,barno) - VertSpacing * eNumber[i] * getPoint(true);         // Plot object VertSpacing pips x seq# below low of relevant bar
    else
      vertpos = iHigh(NULL,0,barno) - VertSpacing * (eNumber[i]+dir1) * getPoint(true);    // Plot object VertSpacing pips x seq# above high of relevant bar
    if (eWingding[i] == 0)  {  
      ObjectCreate(objname,OBJ_TEXT,0,eTime[i],vertpos);                
      ObjectSet(objname,OBJPROP_ANGLE,0);
      ObjectSetText(objname,eText2[i],9,"Verdana",White);                      // Set object descr = second descr line
    } 
    else  {
      ObjectCreate(objname,OBJ_ARROW,0,eTime[i],vertpos);                
      ObjectSet(objname,OBJPROP_ARROWCODE,eWingding[i]);                       // Set wingding code
      ObjectSet(objname,OBJPROP_WIDTH,SymbolSize);                             // Set size of object
      ObjectSetText(objname,eText2[i]);                                        // Set object descr = second descr line
    } 
    ObjectSet(objname,OBJPROP_COLOR,White);                                    // Default color
    if (eImpact[i] == 1)  ObjectSet(objname,OBJPROP_COLOR,Yellow);             // Low impact = yellow
    if (eImpact[i] == 2)  ObjectSet(objname,OBJPROP_COLOR,Orange);             // Medium impact = orange
    if (eImpact[i] == 3)  ObjectSet(objname,OBJPROP_COLOR,Red);                // High impact = red
    if (eImpact[i] == 4)  ObjectSet(objname,OBJPROP_COLOR,LimeGreen);          // Allow other primary colors
    if (eImpact[i] == 5)  ObjectSet(objname,OBJPROP_COLOR,DodgerBlue);        
    if (eImpact[i] == 6)  ObjectSet(objname,OBJPROP_COLOR,Magenta);        
    if (eImpact[i] == 7)  ObjectSet(objname,OBJPROP_COLOR,Aqua);        
    if (eImpact[i] == 8)  ObjectSet(objname,OBJPROP_COLOR,Goldenrod);        
    if (eImpact[i] == 9)  ObjectSet(objname,OBJPROP_COLOR,Plum);        
    if (eImpact[i] == 10) ObjectSet(objname,OBJPROP_COLOR,Silver); 
  }

// Print news events and countdown.......
  if (DisplayPrior > 0 || DisplayNext > 0 || TimeZones)  {
    int yp = yStart;
    for(i=0; i<eCount; i++)  {
      if (StringFind(eText1[i],"\x22 *") > 0)  continue;          // ignore user-defined items

      if (TimeZones && TimeCurrent() < eTime[i] && TimeCurrent() >= eTime[i-1])  {
        yp += 18;
        objname = "newstextd"+i;
        FontColor = TimeZoneColor;
        int xp = xStart + 48;
        string tstr1 = TimeToStr(TimeCurrent(),TIME_DATE) + "        " + TimeToStr(TimeCurrent(),TIME_MINUTES);
        ObjectCreate(objname,OBJ_LABEL,Window,0,0);
        ObjectSet(objname,OBJPROP_CORNER,0);
        ObjectSet(objname,OBJPROP_XDISTANCE,xp);
        ObjectSet(objname,OBJPROP_YDISTANCE,yp);
        ObjectSetText(objname,tstr1,FontSize,FontName,FontColor);

        objname = "newstextc"+i;
        xp = xStart + 210;
        tstr1 =         " (NZ-" + TimeToStr(TimeCurrent()+NZshift*3600,TIME_MINUTES);
        tstr1 = tstr1 + "  AU-" + TimeToStr(TimeCurrent()+AUshift*3600,TIME_MINUTES);
        tstr1 = tstr1 + "  JP-" + TimeToStr(TimeCurrent()+JPshift*3600,TIME_MINUTES);
        tstr1 = tstr1 + "  EU-" + TimeToStr(TimeCurrent()+EUshift*3600,TIME_MINUTES);
        tstr1 = tstr1 + "  GB-" + TimeToStr(TimeCurrent()+GBshift*3600,TIME_MINUTES);
        tstr1 = tstr1 + "  US-" + TimeToStr(TimeCurrent()+USshift*3600,TIME_MINUTES) + ")";
        ObjectCreate(objname,OBJ_LABEL,Window,0,0);
        ObjectSet(objname,OBJPROP_CORNER,0);
        ObjectSet(objname,OBJPROP_XDISTANCE,xp);
        ObjectSet(objname,OBJPROP_YDISTANCE,yp);
        ObjectSetText(objname,tstr1,FontSize,FontName,FontColor);
      }  

      if (eTime[i] < FromDate) continue;                          // Skip if outside entered date range
      if (eTime[i] > ThruDate) continue;
      if (eImpact[i] == 1 && Plot_Low == FALSE)    continue;                     // Skip if impact outside entered values
      if (eImpact[i] == 2 && Plot_Medium == FALSE) continue;
      if (eImpact[i] == 3 && Plot_High == FALSE)   continue;

      if (eWingding[i] == 140 && Plot_USD == FALSE)   continue;                  // Skip if currency outside entered values
      if (eWingding[i] == 141 && Plot_CAD == FALSE)   continue;
      if (eWingding[i] == 142 && Plot_EUR == FALSE)   continue;
      if (eWingding[i] == 143 && Plot_GBP == FALSE)   continue;
      if (eWingding[i] == 144 && Plot_CHF == FALSE)   continue;
      if (eWingding[i] == 145 && Plot_JPY == FALSE)   continue;
      if (eWingding[i] == 146 && Plot_AUD == FALSE)   continue;
      if (eWingding[i] == 147 && Plot_NZD == FALSE)   continue;
      if (eWingding[i] == 148 && Plot_CNY_OTHERS == FALSE)   continue;

      if ( StringSubstr(ePeriods[i],0,1) == "0" && Period() == PERIOD_M1  )   continue;    // Skip if not to be shown on this period's chart
      if ( StringSubstr(ePeriods[i],1,1) == "0" && Period() == PERIOD_M5  )   continue;
      if ( StringSubstr(ePeriods[i],2,1) == "0" && Period() == PERIOD_M15 )   continue;
      if ( StringSubstr(ePeriods[i],3,1) == "0" && Period() == PERIOD_M30 )   continue;
      if ( StringSubstr(ePeriods[i],4,1) == "0" && Period() == PERIOD_H1  )   continue;
      if ( StringSubstr(ePeriods[i],5,1) == "0" && Period() == PERIOD_H4  )   continue;
      if ( StringSubstr(ePeriods[i],6,1) == "0" && Period() == PERIOD_D1  )   continue;
      if ( StringSubstr(ePeriods[i],7,1) == "0" && Period() == PERIOD_W1  )   continue;
      if ( StringSubstr(ePeriods[i],8,1) == "0" && Period() == PERIOD_MN1 )   continue;

      skipflag = FALSE;                                                                    // Skip if currency outside entered values
      if (eCurrency[i] != "ALL")  {
        skipflag = TRUE;
        sym1 = StringSubstr(Symbol(),0,3);
        sym2 = StringSubstr(Symbol(),3,3);
        cmp1 = StringSubstr(eCurrency[i],0,3);
        if (StringLen(eCurrency[i]) > 3)  {
          cmp2 = StringSubstr(eCurrency[i],3,3);
          if ((cmp1 == "ALL" || cmp1 == sym1 || cmp1 == sym2) && (cmp2 == "ALL" || cmp2 == sym1 || cmp2 == sym2)) skipflag = FALSE;   
        }
        else  {
          if (cmp1 == "ALL" || cmp1 == sym1 || cmp1 == sym2) skipflag = FALSE;   
        }  
      }
      if (skipflag == TRUE) continue;  

      if (i > LastEvent - DisplayPrior && i <= LastEvent + DisplayNext)  {
        if (eText1[i] > "")  {
          yp += 18;

          FontColor = White;                                          // same color as chart object
          if (eImpact[i] == 1)   FontColor = Yellow;
          if (eImpact[i] == 2)   FontColor = Orange;
          if (eImpact[i] == 3)   FontColor = Red;
          if (eImpact[i] == 4)   FontColor = LimeGreen;
          if (eImpact[i] == 5)   FontColor = DodgerBlue;
          if (eImpact[i] == 6)   FontColor = Magenta;
          if (eImpact[i] == 7)   FontColor = Aqua;
          if (eImpact[i] == 8)   FontColor = Goldenrod;
          if (eImpact[i] == 9)   FontColor = Plum;
          if (eImpact[i] == 10)  FontColor = Silver;

          objname = "newstextw"+i;                                    // Wingdings symbol
          xp = xStart;
          tstr1 = " ";
          tstr1 = StringSetChar(tstr1,1,eWingding[i]);
          ObjectCreate(objname,OBJ_LABEL,Window,0,0);
          ObjectSet(objname,OBJPROP_CORNER,0);
          ObjectSet(objname,OBJPROP_XDISTANCE,xp);
          ObjectSet(objname,OBJPROP_YDISTANCE,yp-3);
          ObjectSetText("newstextw"+i,tstr1,16,"Wingdings",FontColor);

          objname = "newstext"+i;                                     // Date + countdown minutes
          xp = xStart + 48;
          tstr1 = TimeToStr(eTime[i],TIME_DATE);
          Mins = MathFloor(eTime[i]/60) - MathFloor(TimeCurrent()/60);
          if (Mins > 0)  tstr1 = tstr1 + " [" + Mins + "]";
          ObjectCreate(objname,OBJ_LABEL,Window,0,0);
          ObjectSet(objname,OBJPROP_CORNER,0);
          ObjectSet(objname,OBJPROP_XDISTANCE,xp);
          ObjectSet(objname,OBJPROP_YDISTANCE,yp);
          ObjectSetText(objname,tstr1,FontSize,FontName,FontColor);
          
          objname = "newstexta"+i;                                    // Description line #1
          xp = xStart + 174;
          tstr1 = eText1[i];
          int t = StringFind(tstr1,"\x22 ");
          if (t > 0)  tstr1 = StringSubstr(tstr1,0,12) + " " + StringSubstr(tstr1,13,t-13);
          if (StringLen(tstr1) > 48)   tstr1 = StringSubstr(tstr1,0,48);
          ObjectCreate(objname,OBJ_LABEL,Window,0,0);
          ObjectSet(objname,OBJPROP_CORNER,0);
          ObjectSet(objname,OBJPROP_XDISTANCE,xp);
          ObjectSet(objname,OBJPROP_YDISTANCE,yp);
          ObjectSetText(objname,tstr1,FontSize,FontName,FontColor);

          objname = "newstextb"+i;                                    // Description line #2
//        xp = xStart + 194 + 7*StringLen(tstr1);
          xp = xStart + 520;
          string tstr2 = eText2[i];
          t = StringFind(tstr2,"]");
          if (t > 0)  tstr2 = StringSubstr(tstr2,t+2);  
          ObjectCreate(objname,OBJ_LABEL,Window,0,0);
          ObjectSet(objname,OBJPROP_CORNER,0);
          ObjectSet(objname,OBJPROP_XDISTANCE,xp);
          ObjectSet(objname,OBJPROP_YDISTANCE,yp);
          ObjectSetText(objname,tstr2,FontSize,FontName,FontColor);
          
  } } } }

// Display/sound Alert.......
  for(i=0; i<eCount; i++)   {
    int Mins1 = MathFloor(TimeCurrent()/60);
    int Mins2 = MathFloor(eTime[i]/60);
    int Mins3 = MathFloor(eAlertTime[i]/60);
    if (Mins1 <= Mins2 && Mins1 >= Mins3 && eAlertTime[i] > 0 && StringSubstr(eText1[i],2,1) == ":")  {
      Alert("NEWS: " + eText1[i]);
      eAlertTime[i] = -1;
  } }  
  return(0);
}   

//+------------------------------------------------------------------+
//| delete_objects                                                   |
//+------------------------------------------------------------------+
void delete_objects()  {

  for(int i=ObjectsTotal()-1; i>=0; i--)  {
    string objname = ObjectName(i);
    if (StringLen(objname) < 9)  continue;
    if (ObjectType(objname) == OBJ_LABEL 
    && StringSubstr(objname,0,8) == "newstext")  {
      ObjectDelete(objname);
      continue;
    }
    if (ObjectType(objname) == OBJ_ARROW 
    && StringSubstr(objname,2,1) == ":"
    && StringFind("  ALL USD CAD EUR GBP CHF JPY AUD NZD CNY ",StringSubstr(objname,5,4)) > 0
    && (StringFind(objname,"L \x22") > 0 || StringFind(objname,"M \x22") > 0 || StringFind(objname,"H \x22") > 0))  {
      ObjectDelete(objname);
      continue;
  } }
  if (ObjectFind("Plot_News_FFcal_Reminder") >= 0)  ObjectDelete("Plot_News_FFcal_Reminder");
  return(0);      
}

//+------------------------------------------------------------------+
//| display_reminder                                                 |
//+------------------------------------------------------------------+
void display_reminder()
{
   if (ObjectFind("Plot_News_FFcal_Reminder") >= 0)  ObjectDelete("Plot_News_FFcal_Reminder");
   if (RemindToRunFFcal && ! FLAG_FutureFFEventFound && TimeDayOfWeek(TimeCurrent()) <= 4)
     {
      // If this is not Fri or Sat, then create a Text label reminder to run FFcal.exe
      // The assumption is there will always be at least 1 Friday FF event (usually true), 
      //   so that on Thursday or before, at least one future FF event will be found.
      //   But once Friday starts, you don't insist on finding future events.
      //   and don't issue any reminder until the start-of-the-week (Sunday) or later, up to Thur.
      double pr1 = Low[1] - 10*getPoint(true); // # pips below previous (not the current) bar Low
      ObjectCreate("Plot_News_FFcal_Reminder", OBJ_TEXT, 0, TimeCurrent(), pr1, 0,0,0,0);
      ObjectSetText("Plot_News_FFcal_Reminder", "REMINDER: Run FFcal.exe to get future events", 12, "Arial", Red);
     }
   return(0);
}

double getPoint(bool custommode)
  {
   double point = Point;
   string symbol = Symbol();
   if (! custommode) return(point); // If custommode is False, return regular Point value. If True, continue...
   else
     {
      if (symbol == "NOKJPY" || symbol == "SEKJPY" || symbol == "GBPDKK" 
          || symbol == "GBPNOK" || symbol == "USDSKK" || symbol == "XAG") point = Point; // These are 0.001 on BroCo.
      else if (StringFind(symbol,"JPY",3) == 3 || symbol == "XAUUSD") point = 0.01; // ***JPY, XAUUSD
      else if (StringFind(symbol,"USD",0) >= 0
               || StringFind(symbol,"EUR",0) >= 0
               || StringFind(symbol,"GBP",0) >= 0
               || StringFind(symbol,"CAD",0) >= 0
              ) point = 0.0001;
     }
   return(point);
  }

/*  Remove this line and the similar line below to enable this section
//+------------------------------------------------------------------+
//| stringReplaceEveryMatch                                                 |
//+------------------------------------------------------------------+

string stringReplaceEveryMatch(string str, string toFind, string toReplace) {
    int len = StringLen(toFind);
    int pos = 0;
    string leftPart, rightPart, result = str;
    if (len == 0) return (result); // Cannot find ""
    while (true) {
        // Careful, pos must change each loop or it's an infinite loop.
        pos = StringFind(result, toFind, pos);
        if (pos == -1) {
            break;
        }
        if (pos == 0) {
            leftPart = "";
        } else {
            leftPart = StringSubstr(result, 0, pos);
        }
        pos = pos + len;
        rightPart = StringSubstr(result, pos); 
        result = StringConcatenate(leftPart,toReplace,rightPart);
    }    
    return (result);
} // end of stringReplaceEveryMatch
//+------------------------------------------------------------------+
*/ //Remove this line and the similar line above to enable this section